#SBATCH --partition xeon24
#SBATCH --ntasks 24
#SBATCH --time 1:0:0


import os
import numpy as np
from gpaw import GPAW, PoissonSolver, Mixer
from ase.collections import g2
from gpaw.mpi import world
from gpaw.utilities import h2gpts
from time import time
from ase.optimize import (MDMin, FIRE, LBFGS, LBFGSLineSearch,
                          BFGSLineSearch, BFGS, GoodOldQuasiNewton)
from ase.optimize.berny import Berny

#size = 4
#startrank = world.rank // size * size
#stoprank = startrank + size
#comm = world.new_communicator(np.arange(startrank, stoprank))
comm = world

#opts = ['bfgs', 'bfgsls', 'berny', 'mdmin', 'fire', 'lbfgs', 'lbfgsls', 'oldqn']

opts = ['berny']

origdir = os.getcwd()

starttime = time()

optimizers = dict(bfgs=BFGS,
                  bfgsls=BFGSLineSearch,
                  berny=Berny,
                  mdmin=MDMin,
                  fire=FIRE,
                  lbfgs=LBFGS,
                  lbfgsls=LBFGSLineSearch,
                  oldqn=GoodOldQuasiNewton)

for i, atoms0 in enumerate(g2):
    if len(atoms0) == 1:
        continue
    
    if atoms0.has('initial_magmoms'):
        continue

    atoms0.rattle(seed=i, stdev=0.1)
    atoms0.center(vacuum=4.5)

    for opt in opts:
        dirname = 'out/calc.{:02d}.{}'.format(i, opt)

        proceed = 0
        if comm.rank == 0:
            elapsed = time() - starttime
            if elapsed <= 45 * 60:
                try:
                    os.mkdir(dirname)
                except FileExistsError:
                    pass
                else:
                    proceed = 1

        proceed = comm.sum(proceed)
        if not proceed:
            continue

        try:
            os.chdir(dirname)
            if comm.rank == 0:
                open('unfinished.tmp', 'w').close()
            atoms = atoms0.copy()
            calc = GPAW(mode=PW(600), xc='oldLDA',
                        #gpts=h2gpts(0.2, atoms.cell, idiv=8),
                        communicator=comm,
                        txt='gpaw.txt',
                        mixer=Mixer(0.5, 7, 50.0),
                        convergence=dict(density=1e-8),
                        poissonsolver=PoissonSolver(relax='GS', eps=1e-14))
            atoms.calc = calc

            optcls = optimizers[opt]
            optimizer = optcls(atoms, trajectory='opt.traj', master=comm.rank == 0,
                               logfile='opt.log')

            #try:
            optimizer.run(fmax=0.03, steps=100)
            #except Exception:
            #    continue
            
            if comm.rank == 0:
                os.unlink('unfinished.tmp')
        finally:
            os.chdir(origdir)
